clc
clear
% Matlab program "branch3d.m"
% ctrl+c for exit program
% 'C:\Users\Express\Desktop\matlab1\st001.xlsx'
% 'C:\Users\Express\Desktop\matlab1\1seed.xlsx'
% [0.97 0.96 0.98 0.98 0.1 0.1]
% 3-D invasion percolation code
% This program uses a self-avoiding invasion percolation
% algorithm as described by Stark (1991)
% Anisotropy factors are incorporated using the approach
% of Liu et al, 2004 (WRR)
% As an option - once the invasion percolation is complete (no
% additional growth possible within the user-specified grid),
% a pruning step is performed to remove low (Strahler)
% stream orders
%
% Input files used by this program:
% (1) Initial strength file (optional). Each line in this
% input file should contain NCOL values (not wrap format).
% No breaks between grid layers, so total number of
% lines in the input file is NROW*NLAY.
% As an alternative, a (random) initial strength
% field can be generated by this program.
% (2) Initial seed file (required). This input file should
% have one seed location per line.
% Format is row, column, layer.
%
% Function m-files called by this program:
% (1) mkstr3d.m -- generates initial strengths if requested.
% (2) prune3d.m -- prunes low stream orders if requested.
%
% 2-D networks can be generated with this program by using three
% layers, placing initial seeds in middle layer, and specifying
% the north & south anisotropy factors such that the
% [(minimum strength value) / (anis. factor)] is greater than
% the critical cutoff strength. In this case, all resultant
% channel cells will be in the middle layer, and the top and bottom
% layers can be disregarded.
% Grid dimensions
% disp (' ')
% NROW = input('Enter number of rows in grid, NROW = ');
% NCOL = input('Enter number of columns in grid, NCOL = ');
% NLAY = input('Enter number of layers in grid, NLAY = ');
NROW=200; NCOL=120;NLAY=3;
%.......................................................................
% Initial strength field
% disp(' ')
% disp('Specify method for obtaining the intial strength data.')
% disp('Enter 0 to import from existing file or 1 to generate new strength field.')
%  strength_op = input('Enter your response here = ') ;
  strength_op=0;
if strength_op == 0
%   disp (' ')
%   disp('Enter name of input file containing initial strengths.')
%   disp('(enclose in single quotation marks)')
%   str_file = input('Enter here = ' );
  str_temp = importdata('C:\Users\Express\Desktop\Branchwork\st001.xlsx');
  n = 1;
  for k = 1:NLAY
    for i = 1:NROW
      for j = 1:NCOL
        str(i,j,k) = str_temp(n,j);
      end
      n = n+1;
    end
  end
  clear str_temp;
elseif strength_op ==1             % if requested, call function
  [str] = mkstr3d(NROW,NCOL,NLAY);  % m-file to generate initial
end                                % strength values.
%.........................................................................
% Initial seed file
% disp(' ')
% disp('Enter name of input file containing initial seed locations.')
% disp('(enclose in single quotation marks)')
% seedfile = input('Enter here = ' );
% seeds = importdata(seedfile);
seeds = importdata('C:\Users\Express\Desktop\Branchwork\1seed.xlsx');
% disp (' ')
% disp('Enter anisotropy factors as a row vector.')
% disp('[North South East West Up Down]')
%  anis = input('Enter here = ' );
anis=[0.99 0.98 1 1 0.1 0.1];
% disp(' ')
% disp('Enter the critical cutoff strength.')
cutoff = (103.75842 );
%........................................................................
chan = seeds;
net = zeros(NROW,NCOL,NLAY);    % "net" is 3-D grid that will store
                                % O's (non-channel cells) and l's
                                % (channel cells).
bday = zeros(NROW,NCOL,NLAY);   % "bday" is 3-D grid that stores an
                                % integer date-of-birth for each
                                % channel cell. Non-channel cells
                                % will have bday = 0.
calndr = 1;
%...................................................................... 
% Place initial seed cells into the grid. Also, give the
% seed cells bday = calndr = 1
for p = 1:length(chan(:,1))
  net(chan(p,1),chan(p,2),chan(p,3)) = 1;
  bday(chan(p,1),chan(p,2),chan(p,3)) = calndr;
end
%........................................................................
while chan > 0
    new_growth = 0 ;    % Re-set to collect new channel locations
    calndr = calndr + 1;
%.....................................................................
%.....................................................................
    for q = 1:length(chan(:,1))    % Loop over channel cells that
                                   % represent the most recent growth
        i = chan(q,1); j = chan(q,2); k = chan(q,3); % row,col,lay
                                                     % (source cell)
        nb_vals = 0; nb_data = 0 ;       % intialize variables
%........................................................................
        if j > 1 % look at cells around (i,j-l,k)
           nb_vals = 1 ; % source cell (i,j,k)
           if j > 2
              nb_vals = [nb_vals net(i,j-2,k)];
           end
           if i < NROW
             nb_vals = [nb_vals net(i+1,j-1,k)];
           end
           if i > 1
             nb_vals = [nb_vals net(i-1,j-1,k)];
           end
           if k > 1
             nb_vals = [nb_vals net(i,j-1,k-1)];
           end
           if k < NLAY
             nb_vals = [nb_vals net(i,j-1,k+1)];
           end
           nb_data = [i j-1 k (str(i,j-1,k)/anis(4)) sum(nb_vals)];
         end
         if i < NROW % look at cells around (i+1,j,k)
           nb_vals = 1 ; % source cell (i,j,k)
           if j > 1
             nb_vals = [nb_vals net(i+1,j-1,k)];
           end
           if i < (NROW-1)
             nb_vals = [nb_vals net(i+2,j,k)];
           end
           if j < NCOL
             nb_vals = [nb_vals net(i+1,j+1,k)];
           end
           if k > 1
             nb_vals = [nb_vals net(i + 1, j , k-1)];
           end
           if k < NLAY
             nb_vals = [nb_vals net(i+1,j,k+1)];
           end
           if sum(nb_data) > 0
             nb_data = [nb_data; i+1 j k (str(i+1,j,k)/anis(2)) sum(nb_vals)];
           else
             nb_data = [i+1 j k (str(i+1,j,k)/anis(2)) sum(nb_vals)];
           end
         end
%.............................................................
         if j < NCOL % look at cells around (i,j+1,k)
           nb_vals = 1 ; % source cell (i,j,k)
           if i < NROW
             nb_vals = [nb_vals net(i+1,j+1,k)];
           end
           if j < (NCOL-1)
             nb_vals = [nb_vals net(i,j+2,k)];
           end
           if i > 1
             nb_vals = [nb_vals net(i-1,j+1,k)];
           end
           if k > 1
             nb_vals = [nb_vals net(i,j+1,k-1)];
           end
           if k < NLAY
             nb_vals = [nb_vals net(i,j+1,k+1)];
           end
           if sum(nb_data)>0
             nb_data = [nb_data; i j+1 k (str(i,j+1,k)/anis(3)) sum(nb_vals)];
           else
             nb_data = [i j+1 k (str(i,j+1,k)/anis(3)) sum(nb_vals)];
           end
         end
         %.............................................................
         if i > 1 % look at cells around (i-1,j,k)
           nb_vals = 1 ; % source cell (i,j,k)
           if j > 1
             nb_vals = [nb_vals net(i-1,j-1,k)];
           end
           if j < NCOL
             nb_vals = [nb_vals net(i-1,j+1,k)];
           end
           if i > 2
             nb_vals = [nb_vals net(i-2,j,k)];
           end
           if k > 1
             nb_vals = [nb_vals net(i-1,j,k-1)];
           end
           if k < NLAY
             nb_vals = [nb_vals net(i-1,j,k+1)];
           end
           if sum(nb_data) > 0
             nb_data = [nb_data; i-1 j k (str(i-1,j,k)/anis(1)) sum(nb_vals)];
           else
             nb_data = [i-1 j k (str(i-1,j,k)/anis(1)) sum(nb_vals)];
           end
         end
         %...............................................................
         if k > 1 % look at cells around (i,j,k-l)
           nb_vals = 1 ; % source cell (i,j,k)
           if j > 1
             nb_vals = [nb_vals net(i,j-1,k-1)];
           end
           if i < NROW
             nb_vals = [nb_vals net(i+1,j,k-1)];
           end
           if j < NCOL
             nb_vals = [nb_vals net(i,j+1,k-1)];
           end
           if i > 1
             nb_vals = [nb_vals net(i-1,j,k-1)];
           end
           if k > 2
             nb_vals = [nb_vals net(i,j,k-2)];
           end
           nb_data = [nb_data; i j k-1 (str(i,j,k-1)/anis(6)) sum(nb_vals)];
           end
           %..........................................................
           if k < NLAY % look at cells around (i,j,k+1)
             nb_vals = 1 ; % source cell (i,j,k)
             if j > 1
               nb_vals = [nb_vals net(i,j-1,k+1)];
             end
             if i < NROW
               nb_vals = [nb_vals net(i+1,j,k+1)];
             end
             if j < NCOL
               nb_vals = [nb_vals net(i,j+1,k+1)];
             end
             if i > 1
               nb_vals = [nb_vals net(i-1,j,k+1)];
             end
             if k < (NLAY-1)
               nb_vals = [nb_vals net(i,j,k+2)];
             end
             nb_data = [nb_data; i j k+1 (str(i,j,k+1)/anis(5)) sum(nb_vals) ] ;
             end
             %.........................................................
          nb_data = sortrows(nb_data,4); % sort rows of "nb_data" matrix
                                         % by Column 4, which contains
                                         % the corrected strength data
          num_src_cell_nbs = 0 ; % initialize variable to count
                                 % the new channel cells (growth
                                 % extending from the source
                                 % cell i,j,k)
          for m = 1:length(nb_data(:,1))
            % (nb_data(m,5) == 1) means that source cell is the only
            % neighboring channel cell
            if ( (nb_data(m,5) == 1) & (nb_data(m,4) < cutoff) & ...
               (net(nb_data(m,1),nb_data(m,2),nb_data(m,3)) ==0) )
                net(nb_data(m,1),nb_data(m,2),nb_data(m,3)) = 1;
                bday(nb_data(m,1),nb_data(m,2),nb_data(m,3)) = calndr;
                num_src_cell_nbs = num_src_cell_nbs + 1;
                if new_growth ~= 0 % Store the locations of new channel
                                   % cells in "new_growth" matrix
                  new_growth = [new_growth; nb_data(m,1) nb_data(m,2) nb_data(m,3) ] ;
                else
                  new_growth = [nb_data(m,1) nb_data(m,2) nb_data(m,3)];
                end
            end
            if num_src_cell_nbs ==2 % break terminates this local 'for'
              break                 % loop if maximum allowable
            end                     % neighbors have been invaded
          end
        end            % end of 'for' loop over (current) length of "chan"
    %....................................................................
    %....................................................................
        chan = new_growth;
    end          % end of while loop -- channel development complete
    %...................................................................
    %...................................................................
    %...................................................................
    %...................................................................
    % FINISHED THE INVASION PERCOLATION
    disp (' ')
    disp('Invasion percolation complete. Would you like to prune the network?')
    disp('Enter 0 for No or 1 for Yes')
    prune_op = input('Enter your response here = ') ;
    if prune_op == 0
      disp(' ')
      disp('End of program --no pruning')
      disp(' ')
    elseif prune_op ==1  % call function to prune the network
      [numchnod,chdata,thresh,order,net_out] =prune3d(net,seeds,NROW,NCOL,NLAY);
    disp (' ')
    disp('Pruning complete. Final pruned network saved in "net_out".')
    disp(' ')
  end
%   %......................................................................
%   % Clear temporary program variables
%   clear calndr chan nb_data nb_vals new_growth num_src_cell_nbs p q
% %   strength_op prune_op m n i j k
%   % End of program
% figure; imagesc(net(:,:,2)); axis equal; axis image;
% colormap([ones(1,3); [0 0 1]])
figure; imagesc(order(:,:,2)); axis equal; axis image;
% % figure; imagesc(net_out(:,:,2)); axis equal; axis image;
% % colormap([ones(1,3); [0 0 1]])
figure; imagesc(net_out(:,:,2)); axis equal; axis image;
colormap([ones(1,3); [0 0 1]])


