function [ result ] = GetShortestDistance( line1, line2 )
    SMALL_NUM = 0.00000001;
    
    S1.P0 = line1(1:2);
    S1.P1 = line1(3:4);
    S2.P0 = line2(1:2);
    S2.P1 = line2(3:4);
    
    
    u = S1.P1 - S1.P0;
    v = S2.P1 - S2.P0;
    w = S1.P0 - S2.P0;
    a = dot(u,u);         % always >= 0
    b = dot(u,v);
    c = dot(v,v);         % always >= 0
    d = dot(u,w);
    e = dot(v,w);
    D = a*c - b*b;        % always >= 0
    sD = D;       % sc = sN / sD, default sD = D >= 0
%     sc = D;
%     sN = D;
    tD = D;       % tc = tN / tD, default tD = D >= 0
%     tc = D;
%     tN = D;

    % compute the line parameters of the two closest points
    if (D < SMALL_NUM)  % the lines are almost parallel
        sN = 0;         % force using point P0 on segment S1
        sD = 1;         % to prevent possible division by 0.0 later
        tN = e;
        tD = c;
    
    else                  % get the closest points on the infinite lines
        sN = (b*e - c*d);
        tN = (a*e - b*d);
        if (sN < 0)         % sc < 0 => the s=0 edge is visible
            sN = 0;
            tN = e;
            tD = c;
       
        elseif (sN > sD)   % sc > 1  => the s=1 edge is visible
            sN = sD;
            tN = e + b;
            tD = c;
        end
    end

    if (tN < 0.0)             % tc < 0 => the t=0 edge is visible
        tN = 0.0;
        % recompute sc for this edge
        if (-d < 0)
            sN = 0;
        elseif (-d > a)
            sN = sD;
        else 
            sN = -d;
            sD = a;
        end
    elseif (tN > tD)       % tc > 1  => the t=1 edge is visible
        tN = tD;
        % recompute sc for this edge
        if ((-d + b) < 0)
            sN = 0;
        elseif ((-d + b) > a)
            sN = sD;
        else 
            sN = (-d +  b);
            sD = a;
        end
    end
    % finally do the division to get sc and tc
    if abs(sN) < SMALL_NUM
        sc = 0;
    else
        sc = sN / sD;
    end
    
    if abs(tN) < SMALL_NUM
        tc = 0;
    else
        tc = tN / tD;
    end

    % get the difference of the two closest points
    dP = w + (sc * u) - (tc * v);  % =  S1(sc) - S2(tc)

    result = norm(dP);   % return the closest distance
end

