function [ loops ] = getLoops( distanceMatrix, seed )

    [rowsCount, colsCount] = size(distanceMatrix);
    
    stepHistoryMatrix = cell(rowsCount, colsCount);
    stepHistoryMatrixMinMaxDistance = [];
    
    loops = cell(0);
    
    nextStepTargets = seed;
%     nextStepTargetsHistory = nextStepTargets;
    
    while length(nextStepTargets) > 0
        thisStepIndex = nextStepTargets(1, :);
        
        neighbours = [];
        notEmptyNeighbours = [];
        for eachNeighbour = [0 1; 0 -1; 1 0; -1 0]'
            eachNeighbourIndex = thisStepIndex + eachNeighbour';
            if all(eachNeighbourIndex > 0 & eachNeighbourIndex <= [rowsCount, colsCount])
                if distanceMatrix(eachNeighbourIndex(1), eachNeighbourIndex(2)) > -1
                    neighbours = [neighbours; eachNeighbourIndex];
                    
                    if ~isempty(stepHistoryMatrix{eachNeighbourIndex(1), eachNeighbourIndex(2)})
                        notEmptyNeighbours = [notEmptyNeighbours; eachNeighbourIndex];
                    end
                end
            end
        end
        
        
        isThereAnyLoop = size(notEmptyNeighbours, 1) > 1;
        if isThereAnyLoop
            thisStepHistory = stepHistoryMatrix{thisStepIndex(1), thisStepIndex(2)};
            
            for eachNotEmptyNeighbourInd = 1:size(notEmptyNeighbours, 1)
                eachNotEmptyNeighbour = notEmptyNeighbours(eachNotEmptyNeighbourInd, :);
                neighbourHistory = stepHistoryMatrix{eachNotEmptyNeighbour(1), eachNotEmptyNeighbour(2)};
                
                thisNeighbourIsNotMyParent = thisStepHistory(end-1) ~= neighbourHistory(end);
                if thisNeighbourIsNotMyParent
                    
                    firstCommonStep = 0;
                    for eachThisStepHistory = thisStepHistory(end:-1:1)
                        if ismember(eachThisStepHistory, neighbourHistory)
                            firstCommonStep = eachThisStepHistory;
                            break;
                        end
                    end
                    
                    newLoop = [firstCommonStep setdiff(thisStepHistory, neighbourHistory)...
                        setdiff(neighbourHistory, thisStepHistory) sub2ind([rowsCount, colsCount], thisStepIndex(1), thisStepIndex(2))...
                        sub2ind([rowsCount, colsCount], eachNotEmptyNeighbour(1), eachNotEmptyNeighbour(2))];
                    
                    
                    if ~isAlreadyMemberOfLoops(loops, newLoop)
%                         [rows, cols] = ind2sub([rowsCount, colsCount], newLoop);
%                         loops{end+1} = [rows', cols'];
                        loops{end+1} = newLoop;
                        stepHistoryMatrixMinMaxDistance = [stepHistoryMatrixMinMaxDistance; min(distanceMatrix(newLoop)) max(distanceMatrix(newLoop))];
                        
                        
                        for eachLoop = loops
                            if ~isempty(intersect(newLoop, cell2mat(eachLoop)))
                                
                                subLoop = [setdiff(newLoop, cell2mat(eachLoop)) setdiff(cell2mat(eachLoop), newLoop)];
                                if length(subLoop) > 0 && ~isAlreadyMemberOfLoops(loops, subLoop)
                                    loops{end+1} = subLoop;
                                    stepHistoryMatrixMinMaxDistance = [stepHistoryMatrixMinMaxDistance; min(distanceMatrix(subLoop)) max(distanceMatrix(subLoop))];
                                end

                            end
                        end 
                        
                    end
                end
            end
            
        end
        
        
%         if size(neighbours, 1) < 3
            for eachNeighbourInd = 1:size(neighbours, 1)
                eachNeighbour = neighbours(eachNeighbourInd, :);
                
                if isempty(stepHistoryMatrix{eachNeighbour(1), eachNeighbour(2)}) && any(eachNeighbour ~= seed)
                    stepHistoryMatrix{eachNeighbour(1), eachNeighbour(2)} = ...
                        [cell2mat(stepHistoryMatrix(thisStepIndex(1), thisStepIndex(2))) sub2ind([rowsCount, colsCount], thisStepIndex(1), thisStepIndex(2))];
                   
%                     if all(~ismember(nextStepTargetsHistory, neighbours(eachNeighbourInd, :), 'rows'))
                    nextStepTargets = [nextStepTargets; neighbours(eachNeighbourInd, :)];
%                     end
                    
                end
                
            end
%         else
            
%         end

        nextStepTargets(1, :) = [];
        
    end
    
    
    
    
    
    
    
    [~, sortedMaxLoopsInd] = sort(stepHistoryMatrixMinMaxDistance(:, 2), 'descend');
    [~, sortedMinLoopsInd] = sort(stepHistoryMatrixMinMaxDistance(sortedMaxLoopsInd, 1), 'descend');
    loops = loops(sortedMaxLoopsInd);
    loops = loops(sortedMinLoopsInd);
    
    
%     figure; 
%     test = double(distanceMatrix > -1);
%     for eachSortedLoopIndex = 1:length(loops)
%         eachSortedLoop = loops{eachSortedLoopIndex};
% 
%         test1 = test;
%         test1(eachSortedLoop) = 2;
% 
%         imagesc(test1); axis equal; axis image;
% 
%     end
    
    newLoops = cell(0);
    for eachLoop = loops
        [rows, cols] = ind2sub([rowsCount, colsCount], cell2mat(eachLoop));
        newLoops{end+1} = [rows', cols'];
    end
    loops = newLoops;
    
end

