function [numchnod,chdata,thresh,order,net_out] = prune3d(net,seeds,NROW,NCOL,NLAY)
% Function m-file "prune3d.m"
% Code to assign and remove Strahler stream orders
%  from channel network
% Input matrix (from invasion percoltion code) is a grid of
%  zeros and ones, where the ones identify channel cells 
inpnet = net;            
numchnod = sum(sum(sum(inpnet)));  % report number of channel cells
% First step is to scan through entire grid once
% and create the incidence matrix "chdata"
% Four cell types are identified
%   Type 1 -- only 1 connected cell ("tip"); must be 1st order
%   Type 2 -- 2 connected cells
%   Type 3 -- 3 connected cells, junction
%   Type 4 -- 4 connected cells, junction
%   Type 5 -- initial seed cell (special case)
% Data stored in "chdata"
% chdata(:,1) = row location (y-coord)
% chdata(:,2) = column location (x-coord)
% chdata(:,3) = layer location (z-coord)
% chdata(:,4:6) = row,col,lay of neighborl % always nonzero (unless
% initial seed is a tip)
% chdata(:,7:9) = row,col & lay of neighbor2 % for type 2 or type 3
% chdata(:,10:12) = row,col & lay of neighbor3 % for type 3

% chdata(:,13:15) = row,col & lay of neighbor3 % for type 4
% chdata(:,16) = cell type
% chdata(:,17) = strahler order

% Other variables used to construct "chdata"
% totnghbs = counts the number of connected channel cells
% For every (i,j,k) identified as channel, the code evaluates
% cells (i,j-l), (i+l,j), (i,j+l), (i-l,j), (i,j,k-l), (i,j,k+l)
% in that order to determine if they are also channel cells
% temp1 = row vector that stores locations of neighboring channel cells
% Initialize

initialSeedCellType = 5;
cellTypeColIndex = 16;
strahlerOrderColIndex = 17;
colCount = 18;

originMatrix = zeros([NROW,NCOL]);
originIndex = 1;

chdata = zeros(numchnod,colCount);
index = 1;
% Scan through entire grid containing network
for k = 1:NLAY
  for i = 1:NROW
    for j = 1:NCOL
      if inpnet(i,j,k)==1
        chdata(index,1:3)= [i j k];
        totnghbs = 0;
        temp1 = [];
        
        if j > 1
          if inpnet(i,j-1,k)==1
              temp1=[i j-1 k];
            totnghbs = totnghbs + 1;
          end
        end
        
        if i < NROW
          if inpnet(i+1,j,k) == 1
              temp1 = [temp1 i+1 j k] ;
            totnghbs = totnghbs + 1;
          end
        end
        
        if j < NCOL
          if inpnet(i,j+1,k) == 1
              temp1 = [temp1 i j+1 k] ;
            totnghbs = totnghbs + 1;
          end
        end
        
        if i > 1
          if inpnet(i-1,j,k ) == 1
              temp1 = [temp1 i-1 j k];
            totnghbs = totnghbs + 1;
          end
        end
        
        if k > 1
          if inpnet(i,j,k-1) == 1
              temp1 = [temp1 i j k-1];
            totnghbs = totnghbs + 1;
          end
        end
        
        if k < NLAY
          if inpnet(i,j,k+1) == 1
               temp1 = [temp1 i j k+1];
            totnghbs = totnghbs + 1;
          end
        end
        
        celltype = totnghbs;
%................................................................
% Exit program if a cell has more than three neighbors. This was a
% problem when previous network generator was used for 3-D grids.
      if totnghbs > 4
        disp('row col lay')
        [i j k]
        error('Cell has more than 3 neighbors!')
      end
%..........................................................
         if totnghbs > 0
           chdata(index,4:(3*(celltype + 1))) = temp1;
           chdata(index, cellTypeColIndex) = celltype;
           if celltype == 1
             chdata(index, strahlerOrderColIndex) = 1; % Stream Order is 1 at all tips
             
             originMatrix(i, j) = originIndex;
             originIndex = originIndex + 1;
           end
         else                                          % Note that totngbs = 0 is possible
             chdata(index, strahlerOrderColIndex) = 1; % for initial seed cell that didn't
         end                                           % go anywhere (i.e., also a tip)
         
        index = index + 1;
       end
     end
   end
 end
%...............................................................
% Initial channel seeds are special case where
% (NROW+1) would be a connected cell
% Gather column/layer numbers (x- and z-coordinates) for all
% channel cells along bottom row of the grid
numcells_bot = sum(inpnet(NROW,:));
chan_botrow = zeros(numcells_bot,3);% initialize
index2 = 1;
for q = 1:numchnod
  if chdata(q,1) == NROW
    chan_botrow(index2,1:3) = [q chdata(q,2) chdata(q,3)];
    index2 = index2 + 1;
  end
end
% Now compare these locations stored in chan_botrow
% to the initial seed locations
% When a seed location is identified,
% insert type = 5 into chdata(:,13)
% Also, if seed cell has a neighbor, set the initial
% stream order back to zero (it might have
% been mistakenly identified as a tip)
for i = 1:length(seeds(:,1))
  for j = 1:numcells_bot
    if ((chan_botrow(j,2) == seeds(i,2)) &(chan_botrow(j,3) == seeds(i,3)))
      loc = chan_botrow(j,1);
      chdata(loc,cellTypeColIndex) = initialSeedCellType;
      
      originMatrix(chdata(loc,1),chdata(loc,2)) = 0;
      
      if chdata(loc,4) > 0
        chdata(loc,strahlerOrderColIndex) = 0;
      end
    end
  end
end
%..............................................................
% FINISHED BUILDING THE INCIDENCE MATRIX (chdata)
disp (' ')
disp('Finished initial development of matrix chdata')
disp('Now stream orders will be assigned ...')
disp(' ')
%..............................................................
% Initialize new matrix "order" -- this is size of
% original grid but we never loop through it
order = zeros(NROW,NCOL,NLAY);
% Add first-order locations that are already known (tips)
for q = 1:numchnod
  if chdata(q,cellTypeColIndex) == 1
    order(chdata(q,1),chdata(q,2),chdata(q,3)) = 1;
  end
  if ((chdata(q,cellTypeColIndex) == initialSeedCellType) & (chdata(q,strahlerOrderColIndex) == 1))
    order(chdata(q,1),chdata(q,2),chdata(q,3)) = 1;
  end
end
% Get sum of chdata(:,14) -- stream orders
old_ordersum = sum(chdata(:,strahlerOrderColIndex));
% Now begin computing stream orders for all other
% channel cells (besides tips)

% figure; imagesc(net(:,:,2)); axis equal; axis image;
% colormap([[1 1 1]; [0 1 0]]);


orderToShow = order(:,:,2) - (((order(:,:,2) == 0) & (net(:,:,2) > 0)));
distanceMatrix = getDistanceMatrix(orderToShow, seeds(1:2));
sortedLoops = getLoops(distanceMatrix, seeds(1:2));




figure; 
videoFrames = [];

while min(chdata(:,strahlerOrderColIndex)) == 0
        
    orderToShow = order(:,:,2) - (((order(:,:,2) == 0) & (net(:,:,2) > 0)));
    imagesc(orderToShow); axis equal; axis image;
    colormap([0 0 0; 1 1 1; jet(length( unique(orderToShow) )-2)]);
    
    videoFrames = [videoFrames getframe(gcf)];
    
%     originToShow = originMatrix - (((originMatrix == 0) & (net(:,:,2) > 0)));
%     imagesc(originToShow); axis equal; axis image;
%     colormap([0 0 0; 1 1 1; jet(length( unique(originToShow) )-2)]);
    

    
    for q = 1:numchnod
        if chdata(q,strahlerOrderColIndex) == 0
          
            if chdata(q,cellTypeColIndex) == 2
                  hi_ord = max(order(chdata(q,4),chdata(q,5),chdata(q,6)),order(chdata(q,7),chdata(q,8),chdata(q,9)));
                  order(chdata(q,1),chdata(q,2),chdata(q,3)) = hi_ord;
                  chdata(q,strahlerOrderColIndex) = hi_ord;
                  
                  originMatrix(chdata(q,1),chdata(q,2)) = max(originMatrix(chdata(q,4),chdata(q,5)), originMatrix(chdata(q,7),chdata(q,8)));
    

            elseif chdata(q,cellTypeColIndex) == 3
                  temp2 = [];
                  tempOrigin = [];
                  
                  for i = 4:3:12
                      if order(chdata(q,i),chdata(q,i+1),chdata(q,i+2)) > 0
                          temp2 = [ temp2 order(chdata(q,i),chdata(q,i+1),chdata(q,i+2))];
                          tempOrigin = [tempOrigin originMatrix(chdata(q,i),chdata(q,i+1))];
                      end
                  end
                  

                 % If ready -- assign order to this junction cell
                 if length(temp2) >= 2 % orders known for all upstream neighbs
                   if min(temp2) == max(temp2)
                       
                       if min(tempOrigin) == max(tempOrigin)
                           order(chdata(q,1),chdata(q,2),chdata(q,3)) = temp2(1);
                           chdata(q,strahlerOrderColIndex) = temp2(1);
                           
                           originMatrix(chdata(q,1),chdata(q,2)) = tempOrigin(1);
                       else
                           order(chdata(q,1),chdata(q,2),chdata(q,3)) = temp2(1) + 1;
                           chdata(q,strahlerOrderColIndex) = temp2(1) + 1;
                           
%                            [~, ind] = max(temp2);
                           originMatrix(chdata(q,1),chdata(q,2)) = originIndex;
                           originIndex = originIndex + 1;
                       end

                   else
                       order(chdata(q,1),chdata(q,2),chdata(q,3)) = max(temp2);
                       chdata(q,strahlerOrderColIndex) = max(temp2);
                        
                       [~, ind] = max(temp2);
                       originMatrix(chdata(q,1),chdata(q,2)) = tempOrigin(ind);
%                        originIndex = originIndex + 1;
                   end
                 end
                
            elseif chdata(q,cellTypeColIndex) == 4
                
                 temp2 = [];
                  tempOrigin = [];
                  
                  for i = 4:3:15
                      if order(chdata(q,i),chdata(q,i+1),chdata(q,i+2)) > 0
                          temp2 = [ temp2 order(chdata(q,i),chdata(q,i+1),chdata(q,i+2))];
                          tempOrigin = [tempOrigin originMatrix(chdata(q,i),chdata(q,i+1))];
                      end
                  end


                  

                 % If ready -- assign order to this junction cell
                 if length(temp2) >= 3 % orders known for all upstream neighbs
                   if min(temp2) == max(temp2)
                       
                       if min(tempOrigin) == max(tempOrigin)
                           order(chdata(q,1),chdata(q,2),chdata(q,3)) = temp2(1);
                           chdata(q,strahlerOrderColIndex) = temp2(1);
                           
                           originMatrix(chdata(q,1),chdata(q,2)) = tempOrigin(1);
                       else
                           order(chdata(q,1),chdata(q,2),chdata(q,3)) = temp2(1) + 1;
                           chdata(q,strahlerOrderColIndex) = temp2(1) + 1;
                           
%                            [~, ind] = max(temp2);
                           originMatrix(chdata(q,1),chdata(q,2)) = originIndex;
                           originIndex = originIndex + 1;
                       end

                   else
                       order(chdata(q,1),chdata(q,2),chdata(q,3)) = max(temp2);
                       chdata(q,strahlerOrderColIndex) = max(temp2);
                     
                       [~, ind] = max(temp2);
                       originMatrix(chdata(q,1),chdata(q,2)) = tempOrigin(ind);
%                        originIndex = originIndex + 1;
                   end
                 end
                
                
                % Type 5 cells are last to be assigned for a given tree
                % Note that initial seed could have 0, 1, or 2 neighbors
                % Zero neighbors means initial seed is a tip
                % (order of 1 has already been assigned)
                % Two neighbors means that the initial seed is a junction cell
            elseif ((chdata(q,cellTypeColIndex) == initialSeedCellType) & (chdata(q,strahlerOrderColIndex) ~= 1))
                  if order(chdata(q,4),chdata(q,5),chdata(q, 6)) > 0
                    if chdata(q,7) == 0
                       order(chdata(q,1),chdata(q,2),chdata(q,3)) =order(chdata(q,4),chdata(q,5),chdata(q,6));
                       chdata(q,strahlerOrderColIndex) = order(chdata(q,1),chdata(q,2),chdata(q,3));
                    else
                      if order(chdata(q,7),chdata(q,8),chdata(q,9)) > 0
                        if order(chdata(q,4),chdata(q,5),chdata(q,6)) ==order(chdata(q,7),chdata(q,8),chdata(q,9))
                           order(chdata(q,1),chdata(q,2),chdata(q,3))=order(chdata(q,4),chdata(q,5),chdata(q,6)) + 1;
                           chdata(q,strahlerOrderColIndex) =order(chdata(q,1),chdata(q,2),chdata(q,3));
                        else
                          order(chdata(q,1),chdata(q,2),chdata(q,3)) = max(order(chdata(q,4),chdata(q,5),chdata(q,6)),order(chdata(q,7),chdata(q,8),chdata(q,9)));
                          chdata(q,strahlerOrderColIndex) =order(chdata(q,1),chdata(q,2),chdata(q,3));
                        end
                      end
                    end
                  end
            end
        end 
    end  % end of for loop
    % Exit program if stuck in 'while' loop. Again, this was an
    % occasional problem when previous network generator was used
    % for 3-D grids.
    new_ordersum = sum(chdata(:,strahlerOrderColIndex)) ;
    if new_ordersum == old_ordersum
        orderChanged = 0;
        
        [ is, js, sortedLoops ] = getSortedLoopsBranches( order(:, :, 2), distanceMatrix, sortedLoops );
        

        for eachIndex = 1:length(is)

            [~,q] = ismember([is(eachIndex),js(eachIndex),2],chdata(:, 1:3),'rows');
            for i = 4:3:15
                if chdata(q,i) ~= 0 && chdata(q,i+1) ~= 0
                    if orderToShow(chdata(q,i),chdata(q,i+1)) == -1

                      [~,qNeibour] = ismember([chdata(q,i),chdata(q,i+1),2],chdata(:, 1:3),'rows');
                      [ chdata, originMatrix, order, originIndex, orderChanged ] = ...
                          CalculateLoopOrder( chdata, qNeibour, cellTypeColIndex, originMatrix, originIndex, order, strahlerOrderColIndex, orderChanged );        

                    end
                end
            end 


            if(orderChanged == 1)
                break;
            end

        end
            

    end
    old_ordersum = new_ordersum;
end % end of while loop -- stream orders have
    % been assigned to all channel cells
    

orderToShow = order(:,:,2) - (((order(:,:,2) == 0) & (net(:,:,2) > 0)));
imagesc(orderToShow); axis equal; axis image;
colormap([1 1 1; jet(length( unique(orderToShow) )-1)]);

videoFrames = [videoFrames getframe(gcf)];
CreateVideo(videoFrames);
    
%................................................................
% Remove stream orders as requested by user
disp(' ')
disp('Maximum stream order for this network = ')
max(chdata(:,strahlerOrderColIndex))
disp(' ')
disp('Delete below what order?')
disp('(e.g., reply of 4 would remove streams of order 1,2,and 3)')
thresh = input('Enter threshold order = ');
if thresh < (max(chdata(:, strahlerOrderColIndex)))
  net_out = 1*(order>=thresh) + 0*(order<thresh);
end
%.................................................................
% End of function
end 

     

