function [numchnod,chdata,thresh,order,net_out] = prune3d(net,seeds,NROW,NCOL,NLAY)
% Function m-file "prune3d.m"
% Code to assign and remove Strahler stream orders
%  from channel network
% Input matrix (from invasion percoltion code) is a grid of
%  zeros and ones, where the ones identify channel cells 
inpnet = net;            
numchnod = sum(sum(sum(inpnet)));  % report number of channel cells
% First step is to scan through entire grid once
% and create the incidence matrix "chdata"
% Four cell types are identified
%   Type 1 -- only 1 connected cell ("tip"); must be 1st order
%   Type 2 -- 2 connected cells
%   Type 3 -- 3 connected cells, junction
%   Type 4 -- initial seed cell (special case)
% Data stored in "chdata"
% chdata(:,1) = row location (y-coord)
% chdata(:,2) = column location (x-coord)
% chdata(:,3) = layer location (z-coord)
% chdata(:,4:6) = row,col,lay of neighborl % always nonzero (unless
% initial seed is a tip)
% chdata(:,7:9) = row,col & lay of neighbor2 % for type 2 or type 3
% chdata(:,10:12) = row,col & lay of neighbor3 % for type 3
% chdata(:,13) = cell type
% chdata(:,14) = strahler order
% Other variables used to construct "chdata"
% totnghbs = counts the number of connected channel cells
% For every (i,j,k) identified as channel, the code evaluates
% cells (i,j-l), (i+l,j), (i,j+l), (i-l,j), (i,j,k-l), (i,j,k+l)
% in that order to determine if they are also channel cells
% temp1 = row vector that stores locations of neighboring channel cells
% Initialize
chdata = zeros(numchnod,14);
index = 1;
% Scan through entire grid containing network
for k = 1:3
  for i = 1:325
    for j = 1:200
      if inpnet(i,j,k)==1
        chdata(index,1)=i;
        chdata(index,2)=j;
        chdata(index,3)=k;
        totnghbs = 0;
        temp1 = 0;
        
        if j > 1
          if inpnet(i,j-1,k)==1
              temp1=[i j-1 k];
            totnghbs = totnghbs + 1;
          end
        end
        
        if i < NROW
          if inpnet(i+1,j,k) == 1
            if sum(temp1) > 0
              temp1 = [temp1 i+1 j k] ;
            else
              temp1 = [i+1 j k ];
            end
            totnghbs = totnghbs + 1;
          end
        end
        
        if j < NCOL
          if inpnet(i,j+1,k) == 1
            if sum(temp1) > 0
              temp1 = [temp1 i j+1 k] ;
            else
              temp1 = [i j+1 k] ;
            end
            totnghbs = totnghbs + 1;
          end
        end
        
        if i > 1
          if inpnet(i-1,j,k ) == 1
            if sum(temp1) > 0
              temp1 = [temp1 i-1 j k];
            else
              temp1 =[i-1 j k ];
            end
            totnghbs = totnghbs + 1;
          end
        end
        
        if k > 1
          if inpnet(i,j,k-1) == 1
            if sum(temp1) > 0
              temp1 = [temp1 i j k-1];
            else
              temp1 = [i j k-1] ;
            end
            totnghbs = totnghbs + 1;
          end
        end
        
        if k < NLAY
          if inpnet(i,j,k+1) == 1
            if sum(temp1) > 0
               temp1 = [temp1 i j k+1];
            else
              temp1 = [i j k+1] ;
            end
            totnghbs = totnghbs + 1;
          end
        end
        
        celltype = totnghbs;
%................................................................
% Exit program if a cell has more than three neighbors. This was a
% problem when previous network generator was used for 3-D grids.
      if totnghbs > 4
        disp('row col lay')
        [i j k]
        error('Cell has more than 3 neighbors!')
      end
%..........................................................
         if totnghbs > 0
           chdata(index,4:((3*celltype)+3)) = temp1;
           chdata(index,13) = celltype;
           if celltype == 1
             chdata(index,14) = 1; % Stream Order is 1 at all tips
           end
         else                      % Note that totngbs = 0 is possible
             chdata(index,14) = 1; % for initial seed cell that didn't
         end                       % go anywhere (i.e., also a tip)
         
        index = index + 1;
       end
     end
   end
 end
%...............................................................
% Initial channel seeds are special case where
% (NROW+1) would be a connected cell
% Gather column/layer numbers (x- and z-coordinates) for all
% channel cells along bottom row of the grid
numcells_bot = sum(inpnet(NROW,:));
chan_botrow = zeros(numcells_bot,3);% initialize
index2 = 1;
for q = 1:numchnod
  if chdata(q,1) == NROW
    chan_botrow(index2,1:3) = [q chdata(q,2) chdata(q,3)];
    index2 = index2 + 1;
  end
end
% Now compare these locations stored in chan_botrow
% to the initial seed locations
% When a seed location is identified,
% insert type = 4 into chdata(:,13)
% Also, if seed cell has a neighbor, set the initial
% stream order back to zero (it might have
% been mistakenly identified as a tip)
for i = 1:length(seeds(:,1))
  for j = 1:numcells_bot
    if ((chan_botrow(j,2) == seeds(i,2)) &(chan_botrow(j,3) == seeds(i,3)))
      loc = chan_botrow(j,1);
      chdata(loc,13) = 4;
      if chdata(loc,4) > 0
        chdata(loc,14) = 0;
      end
    end
  end
end
%..............................................................
% FINISHED BUILDING THE INCIDENCE MATRIX (chdata)
disp (' ')
disp('Finished initial development of matrix chdata')
disp('Now stream orders will be assigned ...')
disp(' ')
%..............................................................
% Initialize new matrix "order" -- this is size of
% original grid but we never loop through it
order = zeros(NROW,NCOL,NLAY);
% Add first-order locations that are already known (tips)
for q = 1:numchnod
  if chdata(q,13) == 1
    order(chdata(q,1),chdata(q,2),chdata(q,3)) = 1;
  end
  if ((chdata(q,13) == 4) & (chdata(q,14) == 1))
    order(chdata(q,1),chdata(q,2),chdata(q,3)) = 1;
  end
end
% Get sum of chdata(:,14) -- stream orders
old_ordersum = sum(chdata(:,14));
% Now begin computing stream orders for all other
% channel cells (besides tips)

% figure; imagesc(net(:,:,2)); axis equal; axis image;
% colormap([[1 1 1]; [0 1 0]]);

figure; 

while min(chdata(:,14)) == 0
        
    orderToShow = order(:,:,2) - (((order(:,:,2) == 0) & (net(:,:,2) > 0)));
    imagesc(orderToShow); axis equal; axis image;
    colormap([[0 0 0]; [1 1 1]; [0 1 0]; [1 0 0]; [0 0 1]]);
%     unique(order(:,:,2))
    
    
%     chdata(chdata(:,14) == 0,13)
    
    for q = 1:numchnod
        if chdata(q,14) == 0
          
            if chdata(q,13) == 2
                  hi_ord = max(order(chdata(q,4),chdata(q,5),chdata(q,6)),order(chdata(q,7),chdata(q,8),chdata(q,9)));
                  order(chdata(q,1),chdata(q,2),chdata(q,3)) = hi_ord;
                  chdata(q,14) = hi_ord;

            elseif chdata(q,13) == 3
                  temp2 = 0;


                  if order(chdata(q,4),chdata(q,5),chdata(q,6)) > 0
                    temp2 = order(chdata(q,4),chdata(q,5),chdata(q,6));
                  end


                  if order(chdata(q,7),chdata(q,8),chdata(q,9)) > 0
                    if sum(temp2) > 0
                      temp2 = [temp2 order(chdata(q,7),chdata(q,8),chdata(q,9))];
                    else
                      temp2 = order(chdata(q,7),chdata(q,8),chdata(q,9));
                    end
                  end


                  if order(chdata(q,10),chdata(q,11),chdata(q,12)) > 0
                    if sum(temp2) > 0
                      temp2 = [temp2 order(chdata(q, 10) ,chdata(q, 11) ,chdata(q, 12) ) ];
                    else
                     temp2 = order(chdata(q,10),chdata(q,11),chdata(q,12));
                    end
                  end
                  

                 % If ready -- assign order to this junction cell
                 if length(temp2) ==2 % orders known for all upstream neighbs
                   if min(temp2) == max(temp2)
                     order(chdata(q,1),chdata(q,2),chdata(q,3)) = temp2(1) + 1;
                     chdata(q,14) = temp2(1) + 1;
                   else
                     order(chdata(q,1),chdata(q,2),chdata(q,3)) = max(temp2);
                     chdata(q,14) = max(temp2);
                   end
                 end
                % Type 4 cells are last to be assigned for a given tree
                % Note that initial seed could have 0, 1, or 2 neighbors
                % Zero neighbors means initial seed is a tip
                % (order of 1 has already been assigned)
                % Two neighbors means that the initial seed is a junction cell
            elseif ((chdata(q,13) == 4) & (chdata(q,14) ~= 1))
                  if order(chdata(q,4),chdata(q,5),chdata(q, 6)) > 0
                    if chdata(q,7) == 0
                       order(chdata(q,1),chdata(q,2),chdata(q,3)) =order(chdata(q,4),chdata(q,5),chdata(q,6));
                       chdata(q,14) = order(chdata(q,1),chdata(q,2),chdata(q,3));
                    else
                      if order(chdata(q,7),chdata(q,8),chdata(q,9)) > 0
                        if order(chdata(q,4),chdata(q,5),chdata(q,6)) ==order(chdata(q,7),chdata(q,8),chdata(q,9))
                           order(chdata(q,1),chdata(q,2),chdata(q,3))=order(chdata(q,4),chdata(q,5),chdata(q,6)) + 1;
                           chdata(q,14) =order(chdata(q,1),chdata(q,2),chdata(q,3));
                        else
                          order(chdata(q,1),chdata(q,2),chdata(q,3)) = max(order(chdata(q,4),chdata(q,5),chdata(q,6)),order(chdata(q,7),chdata(q,8),chdata(q,9)));
                          chdata(q,14) =order(chdata(q,1),chdata(q,2),chdata(q,3));
                        end
                      end
                    end
                  end
            end
        end 
    end  % end of for loop
    % Exit program if stuck in 'while' loop. Again, this was an
    % occasional problem when previous network generator was used
    % for 3-D grids.
    new_ordersum = sum(chdata(:,14)) ;
    % if new_ordersum == old_ordersum
    % error('While loop terminated --no progress')
    % end
    old_ordersum = new_ordersum;
end % end of while loop -- stream orders have
    % been assigned to all channel cells
%................................................................
% Remove stream orders as requested by user
disp(' ')
disp('Maximum stream order for this network = ')
max(chdata(:,14))
disp(' ')
disp('Delete below what order?')
disp('(e.g., reply of 4 would remove streams of order 1,2,and 3)')
thresh = input('Enter threshold order = ');
if thresh < (max(chdata(:, 14)))
  net_out = 1*(order>=thresh) + 0*(order<thresh);
end
%.................................................................
% End of function
end 

     

